Package: CoGAPS
Version: 3.33.0
Date: 2025-03-11
Title: Coordinated Gene Activity in Pattern Sets
Author: Jeanette Johnson, Ashley Tsang, Jacob Mitchell, Thomas Sherman, Wai-shing Lee, Conor Kelton, Ondrej Maxian, Jacob Carey,
    Genevieve Stein-O'Brien, Michael Considine, Maggie Wodicka, John Stansfield,
    Shawn Sivy, Carlo Colantuoni, Alexander Favorov, Mike Ochs, Elana Fertig
Description: Coordinated Gene Activity in Pattern Sets (CoGAPS)
    implements a Bayesian MCMC matrix factorization algorithm,
    GAPS, and links it to gene set statistic methods to infer biological
    process activity.  It can be used to perform sparse matrix factorization on
    any data, and when this data represents biomolecules, to do gene set
    analysis.
Maintainer: Elana J. Fertig <ejfertig@jhmi.edu>,
 Thomas D. Sherman <tomsherman159@gmail.com>,
 Jeanette Johnson <jjohn450@jhmi.edu>,
 Dmitrijs Lvovs <dlvovs1@jh.edu>
Depends: R (>= 3.5.0)
Imports: BiocParallel, cluster, methods, gplots, graphics, grDevices,
        RColorBrewer, Rcpp, S4Vectors, SingleCellExperiment, stats,
        SummarizedExperiment, tools, utils, rhdf5, dplyr, fgsea,
        forcats, ggplot2
Suggests: testthat, knitr, rmarkdown, BiocStyle, SeuratObject,
        BiocFileCache, xml2
LinkingTo: Rcpp, BH, testthat
VignetteBuilder: knitr
LazyLoad: true
License: BSD_3_clause + file LICENSE
biocViews: GeneExpression, Transcription, GeneSetEnrichment,
        DifferentialExpression, Bayesian, Clustering, TimeCourse,
        RNASeq, Microarray, MultipleComparison, DimensionReduction,
        ImmunoOncology
NeedsCompilation: yes
RoxygenNote: 7.3.2
Encoding: UTF-8
Collate: 'class-CogapsParams.R' 'CoGAPS.R' 'DistributedCogaps.R'
        'HelperFunctions.R' 'Package.R' 'RcppExports.R' 'SubsetData.R'
        'class-CogapsResult.R' 'data.R' 'methods-CogapsParams.R'
        'methods-CogapsResult.R'
git_url: https://git.bioconductor.org/packages/CoGAPS
git_branch: devel
git_last_commit: 6af7041
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:57:33 UTC; biocbuild
