# Generated by roxygen2: do not edit by hand

S3method(plot,CogapsResult)
export(CoGAPS)
export(CogapsParams)
export(GWCoGAPS)
export(MANOVA)
export(binaryA)
export(buildReport)
export(calcCoGAPSStat)
export(calcZ)
export(checkpointsEnabled)
export(compiledWithOpenMPSupport)
export(findConsensusMatrix)
export(fromCSV)
export(getAmplitudeMatrix)
export(getClusteredPatterns)
export(getCorrelationToMeanPattern)
export(getFeatureLoadings)
export(getMeanChiSq)
export(getOriginalParameters)
export(getParam)
export(getPatternGeneSet)
export(getPatternMatrix)
export(getRetinaSubset)
export(getSampleFactors)
export(getSubsets)
export(getUnmatchedPatterns)
export(getVersion)
export(patternMarkers)
export(plotPatternGeneSet)
export(plotPatternMarkers)
export(plotResiduals)
export(reconstructGene)
export(scCoGAPS)
export(setAnnotationWeights)
export(setDistributedParams)
export(setFixedPatterns)
export(setParam)
export(toCSV)
exportClasses(CogapsParams)
exportClasses(CogapsResult)
import(dplyr)
import(fgsea)
importClassesFrom(S4Vectors,Annotated)
importClassesFrom(S4Vectors,character_OR_NULL)
importClassesFrom(SingleCellExperiment,LinearEmbeddingMatrix)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rcpp,evalCpp)
importFrom(S4Vectors,make_zero_col_DFrame)
importFrom(SummarizedExperiment,assay)
importFrom(cluster,agnes)
importFrom(dplyr,relocate)
importFrom(forcats,fct_reorder)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(gplots,bluered)
importFrom(gplots,heatmap.2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(methods,"slot<-")
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,show)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(methods,validObject)
importFrom(rhdf5,h5read)
importFrom(stats,as.dist)
importFrom(stats,as.hclust)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,hclust)
importFrom(stats,manova)
importFrom(stats,weighted.mean)
importFrom(tools,file_ext)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,write.csv)
useDynLib(CoGAPS)
