#' Toy example to run CoGAPS on.
#'
#'
#' \itemize{
#'   \item V1..V20. some variables, for example levels of gene expression
#' }
#'
#' @docType data
#' @keywords datasets
#' @name modsimdata
#' @usage data(modsimdata)
#' @format 'data.frame':	25 obs. of  20 variables.
NULL

#' Result of applying CoGAPS on the Toy example.
#' 
#'
#' @docType data
#' @keywords datasets
#' @name modsimresult
#' @usage data(modsimresult)
#' @format S4 class ‘CogapsResult’ [package “CoGAPS”] with 7 slots.
NULL