# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

cogaps_from_file_cpp <- function(data, allParams, uncertainty) {
    .Call('_CoGAPS_cogaps_from_file_cpp', PACKAGE = 'CoGAPS', data, allParams, uncertainty)
}

cogaps_cpp <- function(data, allParams, uncertainty) {
    .Call('_CoGAPS_cogaps_cpp', PACKAGE = 'CoGAPS', data, allParams, uncertainty)
}

getBuildReport_cpp <- function() {
    .Call('_CoGAPS_getBuildReport_cpp', PACKAGE = 'CoGAPS')
}

checkpointsEnabled_cpp <- function() {
    .Call('_CoGAPS_checkpointsEnabled_cpp', PACKAGE = 'CoGAPS')
}

compiledWithOpenMPSupport_cpp <- function() {
    .Call('_CoGAPS_compiledWithOpenMPSupport_cpp', PACKAGE = 'CoGAPS')
}

getFileInfo_cpp <- function(path) {
    .Call('_CoGAPS_getFileInfo_cpp', PACKAGE = 'CoGAPS', path)
}

run_catch_unit_tests <- function(reporter = "console") {
    .Call('_CoGAPS_run_catch_unit_tests', PACKAGE = 'CoGAPS', reporter)
}

run_catch_unit_tests_by_tag <- function(tag = "", reporter = "console") {
    .Call('_CoGAPS_run_catch_unit_tests_by_tag', PACKAGE = 'CoGAPS', tag, reporter)
}

