% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Package.R
\docType{data}
\name{GIST.data_frame}
\alias{GIST.data_frame}
\title{GIST gene expression data from Ochs et al. (2009)}
\description{
GIST gene expression data from Ochs et al. (2009)
}
