% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class-CogapsParams.R
\name{initialize,CogapsParams-method}
\alias{initialize,CogapsParams-method}
\title{constructor for CogapsParams}
\usage{
\S4method{initialize}{CogapsParams}(.Object, distributed = NULL, nPatterns, ...)
}
\arguments{
\item{.Object}{CogapsParams object}

\item{distributed}{either "genome-wide" or "single-cell" indicating which
distributed algorithm should be used}

\item{nPatterns}{number of patterns}

\item{...}{initial values for slots}
}
\value{
initialized CogapsParams object
}
\description{
constructor for CogapsParams
}
