% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Package.R
\docType{data}
\name{GIST.result}
\alias{GIST.result}
\title{CoGAPS result from running on GIST dataset}
\description{
CoGAPS result from running on GIST dataset
}
